package com.swagger.rest.controllers

import com.swagger.rest.models.Platform
import com.swagger.rest.models.PlatformInput
import com.swagger.rest.models.Project
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.ProjectRepository
import org.aspectj.asm.internal.ProgramElement.trim
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api/v1")
class PlatformController(
    private val platformRepository: PlatformRepository,
    private val projectRepository: ProjectRepository
) {

    @GetMapping("/platforms")
    fun getPlatformByProjectId(@RequestParam(required = false) project: String?): Any? {
        return try {
            val platforms: List<Platform> =
                if (project == null) {
                    platformRepository.findAll() as List<Platform>
                } else {
                    platformRepository.findByProject(project) as List<Platform>
                }
            if (platforms.isNotEmpty()) {
                ResponseEntity<List<Platform>?>(platforms, HttpStatus.OK)
            } else {
                arrayOf<String>()
            }
        } catch (e: Exception) {
            ResponseEntity<List<Platform>?>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @GetMapping("/platforms/{id}")
    fun getPlatformById(@PathVariable("id") id: Long): ResponseEntity<Platform> {
        val platformData: Optional<Platform?> = platformRepository.findById(id)
        return if (platformData.isPresent) {
            ResponseEntity<Platform>(platformData.get(), HttpStatus.OK)
        } else {
            ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
        }
    }

    @PostMapping("/platforms")
    fun addPlatformByProjectId(
        @RequestBody platform: PlatformInput
    ): ResponseEntity<Platform> {
        return try {
            val found = platformRepository.findByName(platform.name, platform.project_id.toString())!!.size
            val foundProject = projectRepository.findById(platform.project_id!!.toLong()).isPresent
            if (platform.name!!.isNotBlank()) {
                if (!foundProject) {//project not found
                    ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
                } else if (platform.name!!.length > 100) {//too long
                    ResponseEntity<Platform>(HttpStatus.PAYLOAD_TOO_LARGE)
                } else if (found > 0) {//duplicate
                    ResponseEntity<Platform>(HttpStatus.CONFLICT)
                } else {
                    val newPlatform = Platform()
                    newPlatform.name = platform.name!!.trim()
                    newPlatform.project_id = projectRepository.findById(platform.project_id!!).get()
                    ResponseEntity<Platform>(platformRepository.save(newPlatform), HttpStatus.CREATED)
                }
            } else {//invalid name
                ResponseEntity<Platform>(HttpStatus.BAD_REQUEST)
            }
        } catch (e: Exception) {
            ResponseEntity<Platform>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PutMapping("/platforms/{id}")
    fun updatePlatformById(@PathVariable("id") id: Long, @RequestBody input: PlatformInput): Any {
        val targetProject = projectRepository.findById(input.project_id!!.toLong())//target project
        val platformExist = platformRepository.findById(id)//exist data
        return if (input.name!!.isNotBlank()) {
            if (input.name!!.length > 100) {//too long
                ResponseEntity<Platform>(HttpStatus.PAYLOAD_TOO_LARGE)
            } else if (targetProject.isEmpty || platformExist.isEmpty) {//target project not found
                ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
            } else {
                val check = platformRepository.findByName(input.name, input.project_id.toString())
                if (check!!.isEmpty() || (input.name == platformExist.get().name && input.project_id.toString() == platformExist.get().project_id!!.id.toString())) {//tidak ada yg sama
                    val savePlatform = platformExist.get()
                    savePlatform.project_id = targetProject.get()
                    savePlatform.name = input.name!!.trim()
                    ResponseEntity<Platform>(platformRepository.save(savePlatform), HttpStatus.OK)
                } else {
                    ResponseEntity<Platform>(HttpStatus.CONFLICT)
                }
            }
        } else {//invalid name
            ResponseEntity<Platform>(HttpStatus.BAD_REQUEST)
        }
    }

    @DeleteMapping("/platforms/{id}")
    fun deletePlatformById(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val find = platformRepository.findById(id)
        return try {
            if (find.isPresent) {
                platformRepository.deleteById(id)
                ResponseEntity(HttpStatus.OK)
            } else {
                ResponseEntity(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }
}
